import os
import sys
import json
import requests
from typing import Optional
import pytest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from web2json.cli import parse_schema_input, run_pipeline
from web2json.ai_extractor import LLMClient, AIExtractor, OllamaLLMClient
from web2json.preprocessor import BasicPreprocessor
from web2json.postprocessor import PostProcessor
from web2json.pipeline import Pipeline
from web2json.crawler import crawl_urls
from pydantic import BaseModel


class DummyLLM(LLMClient):
    def call_api(self, prompt: str) -> str:
        return json.dumps({"title": "My Title", "content": "Hello"})


def test_parse_schema_input():
    schema_text = "title: str\ncontent: str"
    model = parse_schema_input(schema_text)
    fields = set(model.model_fields.keys())
    assert fields == {"title", "content"}


def test_run_pipeline_with_dummy_llm():
    schema_text = "title: str\ncontent: str"
    content = "<h1>My Title</h1><p>Hello</p>"
    schema_model = parse_schema_input(schema_text)
    llm = DummyLLM()
    extractor = AIExtractor(llm, "{content}\n{schema}")
    pipeline = Pipeline(BasicPreprocessor(), extractor, PostProcessor())
    result = pipeline.run(content, False, schema_model)
    assert result == {"title": "My Title", "content": "Hello"}


def test_parse_schema_input_json_schema():
    schema_json = """
    {
        "type": "object",
        "properties": {
            "title": {"type": "string"},
            "views": {"type": "integer"}
        },
        "required": ["title", "views"]
    }
    """
    model = parse_schema_input(schema_json)
    fields = model.model_fields
    assert set(fields.keys()) == {"title", "views"}
    assert fields["title"].annotation == Optional[str]
    assert fields["views"].annotation == Optional[int]


def test_parse_schema_input_python_class():
    schema_text = """
class Item(BaseModel):
    title: str
    views: int
    """
    model = parse_schema_input(schema_text)
    fields = set(model.model_fields.keys())
    assert fields == {"title", "views"}


def test_run_pipeline_function(monkeypatch):
    def dummy_call_api(self, prompt: str) -> str:
        return json.dumps({"title": "My Title", "content": "Hello"})

    monkeypatch.setattr(OllamaLLMClient, "call_api", dummy_call_api)
    result = run_pipeline(
        "<h1>My Title</h1><p>Hello</p>",
        False,
        "title: str\ncontent: str",
        model_name="dummy"
    )
    assert result == {"title": "My Title", "content": "Hello"}


def test_run_pipeline_missing_field(monkeypatch):
    def dummy_call_api(self, prompt: str) -> str:
        return json.dumps({"title": "My Title"})

    monkeypatch.setattr(OllamaLLMClient, "call_api", dummy_call_api)
    result = run_pipeline(
        "<h1>My Title</h1>",
        False,
        "title: str\ncontent: str\nviews: int",
        model_name="dummy",
    )
    assert result == {"title": "My Title", "content": None, "views": None}


def test_crawl_urls(monkeypatch):
    pages = {
        "https://example.com/docs/": "<a href='/docs/page1.html'>p1</a><a href='/about'>about</a>",
        "https://example.com/docs/page1.html": "<a href='/docs/page2.html'>p2</a>",
        "https://example.com/docs/page2.html": ""
    }

    def fake_get(url, timeout=10):
        class Resp:
            def __init__(self, text):
                self.text = text
        return Resp(pages[url])

    monkeypatch.setattr(requests, "get", fake_get)

    urls = crawl_urls("https://example.com/docs/", max_pages=2)
    assert urls == [
        "https://example.com/docs/",
        "https://example.com/docs/page1.html",
    ]


def test_cli_output_file(tmp_path, monkeypatch):
    def dummy_call_api(self, prompt: str) -> str:
        return json.dumps({"title": "My Title", "content": "Hello"})

    monkeypatch.setattr(OllamaLLMClient, "call_api", dummy_call_api)
    out_file = tmp_path / "out.json"
    argv = [
        "prog",
        "<h1>My Title</h1><p>Hello</p>",
        "--schema",
        "title: str\ncontent: str",
        "--model_name",
        "dummy",
        "--output",
        str(out_file),
    ]
    monkeypatch.setattr(sys, "argv", argv)
    from web2json.cli import main

    main()
    with open(out_file, "r", encoding="utf-8") as fh:
        data = json.load(fh)
    assert data == {"title": "My Title", "content": "Hello"}

def test_run_pipeline_validation_error(monkeypatch):
    def invalid_api(self, prompt: str) -> str:
        return json.dumps({"title": "Hello", "views": "not_an_int"})

    monkeypatch.setattr(OllamaLLMClient, "call_api", invalid_api)
    result = run_pipeline(
        "<h1>Hi</h1>",
        False,
        "title: str\nviews: int",
        model_name="dummy",
    )
    assert result == {"title": "Hello", "views": "not_an_int"}


def test_run_pipeline_fetch_error(monkeypatch):
    def fetch_error(self, url: str):
        raise ValueError("fail")

    monkeypatch.setattr(BasicPreprocessor, "_fetch_content", fetch_error)
    with pytest.raises(ValueError):
        run_pipeline(
            "http://bad", True, "title: str", model_name="dummy"
        )


def test_run_pipeline_without_content_field(monkeypatch):
    created_config = {}

    class DummyPreprocessor(BasicPreprocessor):
        def __init__(self, config=None):
            super().__init__(config)
            created_config.update(self.config)

    def dummy_call_api(self, prompt: str) -> str:
        return json.dumps({"title": "Hello"})

    monkeypatch.setattr(OllamaLLMClient, "call_api", dummy_call_api)
    monkeypatch.setattr(sys.modules["web2json.cli"], "BasicPreprocessor", DummyPreprocessor)

    result = run_pipeline(
        "<h1>Hello</h1>",
        False,
        "title: str",
        model_name="dummy",
    )

    assert result == {"title": "Hello"}
    assert created_config.get("remove_boilerplate") is False

